/*
 * Copyright (c) 2005-, Claybird
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:

 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Claybird nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include "stdafx.h"
#include "extract.h"
#include "ArchiverManager.h"
#include "resource.h"
#include "ConfigCode/ConfigManager.h"
#include "ConfigCode/ConfigGeneral.h"
#include "ConfigCode/ConfigExtract.h"
#include "Utilities/Semaphore.h"
#include "Dialogs/LogDialog.h"
#include "Dialogs/LogListDialog.h"
#include "Dialogs/ProgressDlg.h"
#include "Dialogs/LFFolderDialog.h"
#include "Utilities/StringUtil.h"
#include "Utilities/FileOperation.h"
#include "Utilities/OSUtil.h"
#include "CommonUtil.h"
#include "CmdLineInfo.h"

//폜Ώۂ̋LXg
const LPCWSTR g_szTable=L"0123456789./*-+{}[]@`:;!\"#$%&\'()_><=~^|,\\ @";	//Ō͔p,Sp

//strOpenDir:𓀐JƂAۂɊJpX
bool ExtractOneArchive(CConfigManager &ConfMan,const CConfigGeneral &ConfGeneral,const CConfigExtract &ConfExtract,CArchiverDLL *lpArchiver,LPCTSTR lpArcFile,CPath &r_pathSpecificOutputDir,ARCLOG &r_ArcLog,CPath &r_pathOpenDir)
{
	//t@CL^
	r_ArcLog.strFile=lpArcFile;

	//SȃA[JCuǂA
	//dɃtH_Ȃ悤AɃtH_Kvǂ𔻒肷
	bool bInFolder,bSafeArchive;
	bool bSkipDirCheck=(CREATE_OUTPUT_DIR_SINGLE!=ConfExtract.CreateDir);

	CString strBaseDir;	//dtH_`FbN̂ƂɓASẴt@CtH_̖O()
	CString strExamErr;
	if(!lpArchiver->ExamineArchive(lpArcFile,ConfMan,bSkipDirCheck,bInFolder,bSafeArchive,strBaseDir,strExamErr)){
		//NOTE:B2E32.dll̂߂ɁAG[`FbNキĂ
		ErrorMessage(strExamErr);
		bInFolder=false;
		bSafeArchive=false;
	}

	//A[JCũt@CEtH_̐𒲂ׂ
	int nItemCount=-1;
	if(ConfExtract.CreateNoFolderIfSingleFileOnly){
		//ut@CEtH_̎tH_쐬Ȃvݒ̎ɂ̂ݒ
		nItemCount=lpArchiver->GetFileCount(lpArcFile);
	}

	bool bRet=false;
	CPath pathOutputDir;
	bool bUseForAll=false;	//o̓tH_gȂtrue
	CString strErr;
	HRESULT hr=GetExtractDestDir(lpArcFile,ConfGeneral,ConfExtract,r_pathSpecificOutputDir,bInFolder,pathOutputDir,nItemCount,strBaseDir,r_pathOpenDir,bUseForAll,strErr);
	if(FAILED(hr)){
		if(E_ABORT == hr){
			r_ArcLog.Result=EXTRACT_CANCELED;
			r_ArcLog.strMsg.Format(IDS_ERROR_USERCANCEL);
		}else{
			r_ArcLog.Result=EXTRACT_NG;
			r_ArcLog.strMsg=strErr;
		}
		return false;
	}
	if(bUseForAll){	//̐ݒ㏑
		r_pathSpecificOutputDir=pathOutputDir;
	}

	//o͐fBNgJgfBNgɐݒ
	if(!SetCurrentDirectory(pathOutputDir)){
		r_ArcLog.Result=EXTRACT_NG;
		r_ArcLog.strMsg.Format(IDS_ERROR_CANNOT_SET_CURRENT_DIR,(LPCTSTR)pathOutputDir);
		return false;
	}

	TRACE(_T("Archive Handler Ăяo\n"));
	//------------
	// 𓀂s
	//------------
	if(!lpArchiver->IsUnicodeCapable() && !UtilCheckT2A(pathOutputDir)){
		//UNICODEɑΉĂȂ̂UNICODEt@C̃tH_ɓWJ悤Ƃ
		bRet=false;
		r_ArcLog.Result=EXTRACT_NG;
		r_ArcLog.strMsg=CString(MAKEINTRESOURCE(IDS_ERROR_UNICODEPATH));
	}else{
		CString strLog;
		bRet=lpArchiver->Extract(lpArcFile,ConfMan,ConfExtract,bSafeArchive,pathOutputDir,strLog);

		//---Of[^
		r_ArcLog.Result=(bRet?EXTRACT_OK:EXTRACT_NG);
		r_ArcLog.strMsg=strLog;
	}
	if(ConfGeneral.NotifyShellAfterProcess){
		//𓀊ʒm
		::SHChangeNotify(SHCNE_UPDATEDIR,SHCNF_PATH,pathOutputDir,NULL);
	}
	return bRet;
}

bool DeleteOriginalArchives(const CConfigExtract &ConfExtract,LPCTSTR lpszArcFile)
{
	std::list<CString> fileList;	//폜Ώۂ̃t@Cꗗ

	//---}`{[Ȃ܂Ƃ߂č폜
	CString strFindParam;
	bool bMultiVolume=false;
	if(ConfExtract.DeleteMultiVolume){	//}`{[ł̍폜LH
		bMultiVolume=UtilIsMultiVolume(lpszArcFile,strFindParam);
	}

	CString strFiles;	//t@Cꗗ

	if(bMultiVolume){
		UtilPathExpandWild(fileList,strFindParam);
		for(std::list<CString>::iterator ite=fileList.begin();ite!=fileList.end();++ite){
			strFiles+=_T("\n");
			strFiles+=*ite;
		}
	}else{
		fileList.push_back(lpszArcFile);
	}

	//폜
	if(ConfExtract.MoveToRecycleBin){
		//--------------
		// ݔɈړ
		//--------------
		if(!ConfExtract.DeleteNoConfirm){	//폜mFꍇ
			CString Message;
			if(bMultiVolume){
				//}`{[
				Message.Format(IDS_ASK_MOVE_ARCHIVE_TO_RECYCLE_BIN_MANY);
				Message+=strFiles;
			}else{
				//Pt@C
				Message.Format(IDS_ASK_MOVE_ARCHIVE_TO_RECYCLE_BIN,lpszArcFile);
			}

			//mFS~Ɉړ
			if(IDYES!=MessageBox(NULL,Message,UtilGetMessageCaption(),MB_YESNO|MB_ICONQUESTION|MB_DEFBUTTON2)){
				return false;
			}
		}

		//폜s
		UtilMoveFileToRecycleBin(fileList);
		return true;
	}else{
		//------
		// 폜
		//------
		if(!ConfExtract.DeleteNoConfirm){	//mFꍇ
			CString Message;
			if(bMultiVolume){
				//}`{[
				Message.Format(IDS_ASK_DELETE_ARCHIVE_MANY);
				Message+=strFiles;
			}else{
				//Pt@C
				Message.Format(IDS_ASK_DELETE_ARCHIVE,lpszArcFile);
			}
			if(IDYES!=MessageBox(NULL,Message,UtilGetMessageCaption(),MB_YESNO|MB_ICONQUESTION|MB_DEFBUTTON2)){
				return false;
			}
		}
		//폜s
		for(std::list<CString>::iterator ite=fileList.begin();ite!=fileList.end();++ite){
			DeleteFile(*ite);
		}
		return true;
	}
}


// A[JCutH_pX쐬
void GetArchiveDirectoryPath(const CConfigExtract &ConfExtract,LPCTSTR lpszArcName,CPath &pathDir)
{
	pathDir=lpszArcName;
	pathDir.StripPath();	//pXt@C擾
	pathDir.RemoveExtension();//A[JCut@Cgq폜

	//tH_̐ƋL菜
	if(ConfExtract.RemoveSymbolAndNumber){
		CPath pathOrg=pathDir;
		UtilTrimString(pathDir,g_szTable);
		//ƋL菜ʁA񂪋ɂȂĂ܂Ă猳ɂǂ
		if(_tcslen(pathDir)==0){
			pathDir=pathOrg;
		}
	}

	//󔒂菜
	UtilTrimString(pathDir,_T(".\\ @"));

	if(_tcslen(pathDir)>0){
		pathDir.AddBackslash();
	}
}

/*************************************************************
o͐tH_肷
o͐tH_dɍ쐬Ȃ悤AbInFolderłׂ
tH_ɓĂ邩ǂmF
*************************************************************/
HRESULT GetExtractDestDir(LPCTSTR ArcFileName,const CConfigGeneral &ConfGeneral,const CConfigExtract &ConfExtract,LPCTSTR lpSpecificOutputDir,bool bInFolder,CPath &r_pathOutputDir,const int nItemCount,LPCTSTR lpszBaseDir,CPath &r_pathOpenDir,bool &r_bUseForAll/*ȍ~̃tH_ɉ𓀂ȂtrueԂ*/,CString &strErr)
{
	CPath pathOutputDir;

	if(lpSpecificOutputDir && 0!=_tcslen(lpSpecificOutputDir)){
		//fBNgo͐ƂĎw肳Ăꍇ
		pathOutputDir=lpSpecificOutputDir;
	}else{
		//ݒɏo͐߂
		HRESULT hr=GetOutputDirPathFromConfig(ConfExtract.OutputDirType,ArcFileName,ConfExtract.OutputDir,pathOutputDir,r_bUseForAll,strErr);
		if(FAILED(hr)){
			return hr;
		}
	}
	pathOutputDir.AddBackslash();

	// o͐悪lbg[NhCu/[oufBXNłȂx
	// o͐悪݂ȂȂA쐬mF
	HRESULT hStatus=ConfirmOutputDir(ConfGeneral,pathOutputDir,strErr);
	if(FAILED(hStatus)){
		//LZȂ
		return hStatus;
	}

	//A[JCutH_
	CPath pathArchiveNamedDir;	//A[JCũtH_
	bool bCreateArchiveDir=false;	//A[JCũtH_쐬ꍇɂtrue
	if(
		(!ConfExtract.CreateNoFolderIfSingleFileOnly || nItemCount!=1)&&
			(
				((CREATE_OUTPUT_DIR_SINGLE==ConfExtract.CreateDir)&&!bInFolder)||
				 (CREATE_OUTPUT_DIR_ALWAYS==ConfExtract.CreateDir)
			)
	){
		GetArchiveDirectoryPath(ConfExtract,ArcFileName,pathArchiveNamedDir);
		if(_tcslen(pathArchiveNamedDir)==0){
			hStatus=S_FALSE;	//NG
		}else{
			pathOutputDir+=pathArchiveNamedDir;
			bCreateArchiveDir=true;
		}
	}

	//pX̒`FbN
	if(S_OK==hStatus){
		if(_tcslen(pathOutputDir)>=_MAX_PATH){
			// pXƂ
			//TODO
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_MAX_PATH)));
			hStatus=S_FALSE;
		}
	}

	//̂܂܂ł̓tH_gȂꍇ
	if(S_FALSE==hStatus){
		// Otĕۑ
		CString title(MAKEINTRESOURCE(IDS_INPUT_TARGET_FOLDER_WITH_SHIFT));
		CLFFolderDialog dlg(NULL,title,BIF_RETURNONLYFSDIRS|BIF_NEWDIALOGSTYLE);
		if(IDOK==dlg.DoModal()){
			r_bUseForAll=(GetKeyState(VK_SHIFT)<0);	//TODO
			pathOutputDir=dlg.GetFolderPath();
			pathOutputDir.AddBackslash();
			if(bCreateArchiveDir){
				pathOutputDir+=pathArchiveNamedDir;
			}
		}else{
			//LZ
			return E_ABORT;
		}
	}

	//o͐(A[JCuƓ̃tH_܂߂)tH_݂邱Ƃۏ؂
	if(!UtilMakeSureDirectoryPathExists(pathOutputDir)){
		strErr.Format(IDS_ERROR_CANNOT_MAKE_DIR,(LPCTSTR)pathOutputDir);
		return E_FAIL;
	}

	//o͐tH_Ԃ
	r_pathOutputDir=pathOutputDir;

	//JpX̑gݗ
	CPath pathToOpen=pathOutputDir;
	if(!bCreateArchiveDir){	//A[JCutH_쐬Ȃꍇ
		if(ConfExtract.CreateNoFolderIfSingleFileOnly && nItemCount==1){
			//-t@CEtH_ł邽߃tH_쐬Ȃ
			//Nothing to do
		}else if(CREATE_OUTPUT_DIR_SINGLE==ConfExtract.CreateDir && bInFolder){
			//-dɃtH_Ȃݒɏ]AtH_쐬Ȃ
			//A[JCutH_J
			pathToOpen.Append(lpszBaseDir);
			pathToOpen.AddBackslash();
		}
	}
	pathToOpen.QuoteSpaces();
	//JpXԂ
	r_pathOpenDir=pathToOpen;
	return true;
}

bool Extract(std::list<CString> &ParamList,CConfigManager &ConfigManager,DLL_ID ForceDLL,LPCTSTR lpSpecificOutputDir,const CMDLINEINFO* lpCmdLineInfo)
{
	TRACE(_T("Function ::Extract() started.\n"));
	CConfigGeneral ConfGeneral;
	CConfigExtract ConfExtract;

	ConfGeneral.load(ConfigManager);
	ConfExtract.load(ConfigManager);

	//ݒ㏑
	if(lpCmdLineInfo){
		if(-1!=lpCmdLineInfo->OutputToOverride){
			ConfExtract.OutputDirType=lpCmdLineInfo->OutputToOverride;
		}
		if(-1!=lpCmdLineInfo->CreateDirOverride){
			ConfExtract.CreateDir=lpCmdLineInfo->CreateDirOverride;
		}
		if(-1!=lpCmdLineInfo->DeleteAfterProcess){
			ConfExtract.DeleteArchiveAfterExtract=lpCmdLineInfo->DeleteAfterProcess;
		}
	}

	//Z}tHɂr
	CSemaphoreLocker SemaphoreLock;
	if(ConfExtract.LimitExtractFileCount){
		SemaphoreLock.Lock(LHAFORGE_EXTRACT_SEMAPHORE_NAME,ConfExtract.MaxExtractFileCount);
	}

	UINT uFiles=ParamList.size();	//ɂt@C̐

	//vOXo[
	CProgressDialog dlg;
	//bZ[W[v񂷂߂̃^C}[
	int timer=NULL;
	if(uFiles>=2){	//t@C鎞Ɍ
		dlg.Create(NULL);
		dlg.SetTotalFileCount(uFiles);
		dlg.ShowWindow(SW_SHOW);
		timer=SetTimer(NULL,NULL,1000,UtilMessageLoopTimerProc);
	}

	//w̏o͐
	CPath pathSpecificOutputDir(lpSpecificOutputDir ? lpSpecificOutputDir : _T(""));

	std::vector<ARCLOG> LogArray;	//ʂێ
	bool bAllOK=true;	//ׂĖȂ𓀂true

	//𓀏
	for(std::list<CString>::iterator ite_param=ParamList.begin();ite_param!=ParamList.end();++ite_param){
		//vOXo[i߂
		if(dlg.IsWindow())dlg.SetNextState(*ite_param);

		ARCLOG arcLog;

		//bZ[W[v
		while(UtilDoMessageLoop())continue;

		//A[JConh擾
		//UNICODEΉDLLɃjR[ht@CnƂꍇ͒eBāAł͎š𖾂łȂ
		CArchiverDLL *lpArchiver=CArchiverDLLManager::GetInstance().GetArchiver(*ite_param,ConfExtract.DenyExt,ForceDLL);
		if(!lpArchiver){
			//ΉnhȂ
			arcLog.Result=EXTRACT_NOTARCHIVE;
			arcLog.strMsg.Format(IDS_ERROR_ILLEGAL_HANDLER,(LPCTSTR)*ite_param);
			arcLog.strFile=*ite_param;
			bAllOK=false;
			LogArray.push_back(arcLog);
			continue;
		}

		CPath pathOpenDir;		//t@CJׂtH_
		//𓀎s
		bool bRet=ExtractOneArchive(ConfigManager,ConfGeneral,ConfExtract,lpArchiver,*ite_param,pathSpecificOutputDir,arcLog,pathOpenDir);
		//Oۑ
		LogArray.push_back(arcLog);

		if(!bRet){
			bAllOK=false;
		}else{
			//o͐tH_J
			if(ConfExtract.OpenDir){
				if(ConfGeneral.Filer.UseFiler){
					//p[^WJɕKvȏ
					std::map<stdString,CString> envInfo;
					MakeExpandInformationEx(envInfo,pathOpenDir,NULL);

					//R}hEp[^WJ
					CString strCmd,strParam;
					UtilExpandTemplateString(strCmd,ConfGeneral.Filer.FilerPath,envInfo);	//R}h
					UtilExpandTemplateString(strParam,ConfGeneral.Filer.Param,envInfo);	//p[^
					ShellExecute(NULL, _T("open"), strCmd,strParam, NULL, SW_SHOWNORMAL);
				}else{
					//ExplorerŊJ
					UtilNavigateDirectory(pathOpenDir);
				}
			}

			//ɉ𓀂łkt@C폜orݔɈړ
			if(bRet && ConfExtract.DeleteArchiveAfterExtract){
				if(!ConfExtract.ForceDelete && lpArchiver->IsWeakErrorCheck()){
					//G[`FbN@\nȂ߁A𓀎sɂƔfĂ܂悤
					//DLLgƂɂ͖IɎw肵Ȃ폜Ȃ
					MessageBox(NULL,CString(MAKEINTRESOURCE(IDS_MESSAGE_EXTRACT_DELETE_SKIPPED)),UtilGetMessageCaption(),MB_OK|MB_ICONINFORMATION);
				}else{
					//폜
					DeleteOriginalArchives(ConfExtract,*ite_param);
				}
			}
		}
	}
	//vOXo[
	if(dlg.IsWindow())dlg.DestroyWindow();
	//^C}[
	if(timer)KillTimer(NULL,timer);

	//---O\
	switch(ConfGeneral.LogViewEvent){
	case LOGVIEW_ON_ERROR:
		if(!bAllOK){
			if(1==uFiles){
				//t@C̎̓_CAO{bNX
				if(EXTRACT_CANCELED!=LogArray[0].Result){
					ErrorMessage(LogArray[0].strMsg);
				}
			}else{
				//Oɕ\
				CLogListDialog LogDlg(CString(MAKEINTRESOURCE(IDS_LOGINFO_OPERATION_EXTRACT)));
				LogDlg.SetLogArray(LogArray);
				LogDlg.DoModal(::GetDesktopWindow());
			}
		}
		break;
	case LOGVIEW_ALWAYS:
		//Oɕ\
		CLogListDialog LogDlg(CString(MAKEINTRESOURCE(IDS_LOGINFO_OPERATION_EXTRACT)));
		LogDlg.SetLogArray(LogArray);
		LogDlg.DoModal(::GetDesktopWindow());
		break;
	}

	TRACE(_T("Exit Extract()\n"));
	return bAllOK;
}

